@@ -2,9 +2,9 @@
  * This is the Qt Designer plugin that collects all the Python plugins it can
  * find as a widget collection to Designer.
  *
- * Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
  * 
- * This file is part of PyQt5.
+ * This file is part of PyQt6.
  * 
  * This file may be used under the terms of the GNU General Public License
  * version 3.0 as published by the Free Software Foundation and appearing in
@@ -28,17 +28,18 @@
 #include <QtGlobal>
 #include <QtPlugin>
 #include <QCoreApplication>
-#include <QDesignerCustomWidgetInterface>
 #include <QDir>
 #include <QLibrary>
 #include <QStringList>
 #include <QVector>
+#include <QtDesigner>
 
 #include "../qpy/QtDesigner/qpydesignercustomwidgetplugin.h"
 
 
 // Construct the collection of Python widgets.
-PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent)
+PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent),
+        sys_path(0), sip_unwrapinstance(0), qtdesigner_custom(0)
 {
     // Get the default list of directories to search.  These correspond to a
     // standard "python" subdirectory of all the places that Designer looks for
@@ -79,15 +80,6 @@ PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent)
     else
         dirs = default_dirs;
 
-    // The sys.path object if we need it.
-    PyObject *sys_path = 0;
-
-    // The sip.unwrapinstance object if we need it.
-    PyObject *sip_unwrapinstance = 0;
-
-    // The PyQt5.QtDesigner.QPyDesignerCustomWidgetPlugin object if we need it.
-    PyObject *qtdesigner_custom = 0;
-
     // Go through each directory.
     for (int i = 0; i < dirs.size(); ++i)
     {
@@ -134,155 +126,176 @@ PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent)
             if (!library.load())
                 return;
 
-            Py_Initialize();
+            PyConfig py_config;
+            PyConfig_InitPythonConfig(&py_config);
+
+            // If we seem to be running in a venv the set the program name
+            // explicitly so that Python finds the right site-packages.
+            QString venv = QString::fromLocal8Bit(qgetenv("VIRTUAL_ENV"));
+
+            if (!venv.isEmpty())
+            {
+                venv.append(QDir::separator());
+#if defined(Q_OS_WIN)
+                venv.append(QLatin1String("Scripts"));
+#else
+                venv.append(QLatin1String("bin"));
+#endif
+                venv.append(QDir::separator()).append(QLatin1String("python"));
+
+                wchar_t *venv_wc = new wchar_t[venv.length() + 1];
+                venv_wc[venv.toWCharArray(venv_wc)] = L'\0';
+
+                // Note that it's not clear if the string can be garbage
+                // collected.
+                py_config.program_name = venv_wc;
+            }
+
+            PyStatus status = Py_InitializeFromConfig(&py_config);
+
+            if (PyStatus_Exception(status))
+                return;
+
+            PyConfig_Clear(&py_config);
+
+#ifdef WITH_THREAD
+            // Make sure we don't have the GIL.
+            PyEval_SaveThread();
+#endif
         }
 
-        // Make sure we have sys.path.
+        // Import the plugins with the GIL held.
+#if defined(WITH_THREAD)
+        PyGILState_STATE gil_state = PyGILState_Ensure();
+#endif
+
+        bool fatal = importPlugins(dir, plugins);
+
+#if defined(WITH_THREAD)
+        PyGILState_Release(gil_state);
+#endif
+
+        if (fatal)
+            break;
+    }
+}
+
+
+// Import the plugins from a directory.
+bool PyCustomWidgets::importPlugins(const QString &dir, const QStringList &plugins)
+{
+    // Make sure we have sys.path.
+    if (!sys_path)
+    {
+        sys_path = getModuleAttr("sys", "path");
+
         if (!sys_path)
-        {
-            sys_path = getModuleAttr("sys", "path");
+            return true;
+    }
 
-            if (!sys_path)
-                return;
-        }
+    // Make sure we have sip.unwrapinstance.
+    if (!sip_unwrapinstance)
+    {
+        sip_unwrapinstance = getModuleAttr("PyQt6.sip", "unwrapinstance");
 
-        // Make sure we have sip.unwrapinstance.
         if (!sip_unwrapinstance)
-        {
-            sip_unwrapinstance = getModuleAttr("sip", "unwrapinstance");
+            return true;
+    }
 
-            if (!sip_unwrapinstance)
-                return;
-        }
+    // Convert the directory to a Python object with native separators.
+    QString native_dir = QDir::toNativeSeparators(dir);
 
-        // Convert the directory to a Python object with native separators.
-        dir = QDir::toNativeSeparators(dir);
+    PyObject *dobj = PyUnicode_FromKindAndData(PyUnicode_2BYTE_KIND, native_dir.constData(), native_dir.length());
 
-#if PY_VERSION_HEX >= 0x03030000
-        PyObject *dobj = PyUnicode_FromKindAndData(PyUnicode_2BYTE_KIND, dir.constData(), dir.length());
+    if (!dobj)
+    {
+        PyErr_Print();
+        return false;
+    }
 
-        if (!dobj)
-        {
-            PyErr_Print();
-            continue;
-        }
-#elif PY_VERSION_HEX >= 0x03000000
-#if defined(Py_UNICODE_WIDE)
-        QVector<uint> ucs4 = dir.toUcs4();
+    // Add the directory to sys.path.
+    int rc = PyList_Append(sys_path, dobj);
+    Py_DECREF(dobj);
 
-        PyObject *dobj = PyUnicode_FromUnicode(0, ucs4.size());
+    if (rc < 0)
+    {
+        PyErr_Print();
+        return false;
+    }
 
-        if (!dobj)
+    // Import each plugin.
+    for (int plug = 0; plug < plugins.size(); ++plug)
+    {
+        PyObject *plug_mod = PyImport_ImportModule(plugins.at(plug).toLatin1().data());
+
+        if (!plug_mod)
         {
             PyErr_Print();
             continue;
         }
 
-        memcpy(PyUnicode_AS_UNICODE(dobj), ucs4.constData(),
-                ucs4.size() * sizeof (Py_UNICODE));
-#else
-        PyObject *dobj = PyUnicode_FromUnicode(0, dir.length());
-
-        if (!dobj)
+        // Make sure we have QPyDesignerCustomWidgetPlugin.  We make sure this
+        // is after the import of the first plugin to allow that plugin to
+        // change any API versions.
+        if (!qtdesigner_custom)
         {
-            PyErr_Print();
-            continue;
+            qtdesigner_custom = getModuleAttr("PyQt6.QtDesigner", "QPyDesignerCustomWidgetPlugin");
+
+            if (!qtdesigner_custom)
+                return true;
         }
 
-        memcpy(PyUnicode_AS_UNICODE(dobj), dir.utf16(),
-                dir.length() * sizeof (Py_UNICODE));
-#endif
-#else
-        PyObject *dobj = PyString_FromString(dir.toLatin1().constData());
+        // Go through the module looking for types that implement
+        // QDesignerCustomWidgetInterface (ie. by deriving from
+        // QPyDesignerCustomWidgetPlugin).
+        PyObject *mod_dict = PyModule_GetDict(plug_mod);
+        PyObject *key, *value;
+        Py_ssize_t pos = 0;
 
-        if (!dobj)
+        while (PyDict_Next(mod_dict, &pos, &key, &value))
         {
-            PyErr_Print();
-            continue;
-        }
-#endif
+            if (!PyType_Check(value))
+                continue;
 
-        // Add the directory to sys.path.
-        int rc = PyList_Append(sys_path, dobj);
-        Py_DECREF(dobj);
+            if (value == qtdesigner_custom)
+                continue;
 
-        if (rc < 0)
-        {
-            PyErr_Print();
-            continue;
-        }
+            if (!PyType_IsSubtype((PyTypeObject *)value, (PyTypeObject *)qtdesigner_custom))
+                continue;
 
-        // Import each plugin.
-        for (int plug = 0; plug < plugins.size(); ++plug)
-        {
-            PyObject *plug_mod = PyImport_ImportModule(plugins.at(plug).toLatin1().data());
+            // Create the plugin instance.  Note that we don't give it a
+            // parent, which make things easier.  It also means that Python
+            // owns the instance so we don't decrement the reference count so
+            // that it doesn't get garbage collected.
+            PyObject *plugobj = PyObject_CallObject(value, NULL);
 
-            if (!plug_mod)
+            if (!plugobj)
             {
                 PyErr_Print();
                 continue;
             }
 
-            // Make sure we have QPyDesignerCustomWidgetPlugin.  We make sure
-            // this is after the import of the first plugin to allow that
-            // plugin to change any API versions.
-            if (!qtdesigner_custom)
-            {
-                qtdesigner_custom = getModuleAttr("PyQt5.QtDesigner", "QPyDesignerCustomWidgetPlugin");
+            // Get the address of the C++ instance.
+            PyObject *plugaddr = PyObject_CallFunctionObjArgs(sip_unwrapinstance, plugobj, NULL);
 
-                if (!qtdesigner_custom)
-                    return;
+            if (!plugaddr)
+            {
+                Py_DECREF(plugobj);
+                PyErr_Print();
+                continue;
             }
 
-            // Go through the module looking for types that implement
-            // QDesignerCustomWidgetInterface (ie. by deriving from
-            // QPyDesignerCustomWidgetPlugin).
-            PyObject *mod_dict = PyModule_GetDict(plug_mod);
-            PyObject *key, *value;
-            Py_ssize_t pos = 0;
+            void *addr = PyLong_AsVoidPtr(plugaddr);
+            Py_DECREF(plugaddr);
 
-            while (PyDict_Next(mod_dict, &pos, &key, &value))
-            {
-                if (!PyType_Check(value))
-                    continue;
-
-                if (value == qtdesigner_custom)
-                    continue;
-
-                if (!PyType_IsSubtype((PyTypeObject *)value, (PyTypeObject *)qtdesigner_custom))
-                    continue;
-
-                // Create the plugin instance.  Note that we don't give it a
-                // parent, which make things easier.  It also means that Python
-                // owns the instance so we don't decrement the reference count
-                // so that it doesn't get garbage collected.
-                PyObject *plugobj = PyObject_CallObject(value, NULL);
-
-                if (!plugobj)
-                {
-                    PyErr_Print();
-                    continue;
-                }
-
-                // Get the address of the C++ instance.
-                PyObject *plugaddr = PyObject_CallFunctionObjArgs(sip_unwrapinstance, plugobj, NULL);
-
-                if (!plugaddr)
-                {
-                    Py_DECREF(plugobj);
-                    PyErr_Print();
-                    continue;
-                }
-
-                void *addr = PyLong_AsVoidPtr(plugaddr);
-                Py_DECREF(plugaddr);
-
-                widgets.append(reinterpret_cast<QPyDesignerCustomWidgetPlugin *>(addr));
-            }
-
-            Py_DECREF(plug_mod);
+            widgets.append(reinterpret_cast<QPyDesignerCustomWidgetPlugin *>(addr));
         }
+
+        Py_DECREF(plug_mod);
     }
+
+    // No fatal errors.
+    return false;
 }
 
 
