@@ -1,8 +1,8 @@
 // This is the interface of the Chimera and related classes.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -27,6 +27,7 @@
 #include <QByteArray>
 #include <QHash>
 #include <QList>
+#include <QMetaType>
 #include <QMetaProperty>
 #include <QVariant>
 
@@ -64,7 +65,8 @@ public:
     static ToQVariantConvertors registeredToQVariantConvertors;
 
     // Register a function for converting a Python object to QVariant data.
-    typedef bool (*ToQVariantDataConvertorFn)(PyObject *, void *, int, bool *);
+    typedef bool (*ToQVariantDataConvertorFn)(PyObject *, void *, QMetaType,
+            bool *);
     typedef QList<ToQVariantDataConvertorFn> ToQVariantDataConvertors;
     static ToQVariantDataConvertors registeredToQVariantDataConvertors;
 
@@ -132,7 +134,7 @@ public:
             const char *context);
 
     // Raise an exception after parse() has failed.
-    static void raiseParseException(PyObject *type, const char *context);
+    static void raiseParseException(PyObject *type, const char *context = 0);
 
     class Storage
     {
@@ -207,10 +209,6 @@ public:
     // Convert a QVariant to a Python object based on the type of the object.
     static PyObject *toAnyPyObject(const QVariant &var);
 
-    // Returns the Qt meta-type id.  It will be QMetaType::UnknownType if the
-    // type isn't known to Qt's meta-type system.
-    int metatype() const {return _metatype;}
-
     // Returns a borrowed reference to the Python type object that was used to
     // define the type (if any).
     PyObject *py_type() const {return (PyObject *)_py_type;}
@@ -218,17 +216,22 @@ public:
     // Returns the C++ name of the type.
     const QByteArray &name() const {return _name;}
 
-    // Returns true if the type is an enum.
-    bool isEnum() const {return (_type && sipTypeIsEnum(_type));}
-
-    // Returns true if the type is a flag.
-    bool isFlag() const {return _is_flag;}
+    // Returns true if the type is a C++ or Python enum or flag.
+    bool isEnumOrFlag() const;
 
     // Returns the SIP generated type structure.
     const sipTypeDef *typeDef() const {return _type;}
 
-    // Register a type as an int type.
-    static void registerIntType(PyObject *int_type);
+    // Register a type as a Python enum.
+    static void registerPyEnum(PyObject *enum_type,
+            const QByteArray &fq_cpp_name);
+
+    // The Qt meta-type.
+    QMetaType metatype;
+
+    // The type identifer.  This will be the meta-type identifier or -1 if the
+    // type is char *.
+    int typeId() const {return _is_char_star ? -1 : metatype.id();}
 
 private:
     // The generated type structure.  This may be 0 if the type is known by Qt
@@ -239,21 +242,20 @@ private:
     // was given.
     PyTypeObject *_py_type;
 
-    // The Qt meta-type.  This will always be valid but may refer to the
-    // PyObject wrapper if the underlying type isn't known to Qt.
-    int _metatype;
-
     // Set if the meta-type needn't be exact.
     bool _inexact;
 
     // Set if the the type is derived from QFlags.
-    bool _is_flag;
+    bool _is_qflags;
 
     // The C++ name of the type.
     QByteArray _name;
 
-    // The registered int types.
-    static QList<PyObject *> _registered_int_types;
+    // Set if the C++ type is char *.  This needs special handling.
+    bool _is_char_star;
+
+    // The registered user-defined enums.
+    static QHash<PyObject *, QByteArray> _py_enum_types;
 
     // The cache of previously parsed argument type lists.
     static QHash<QByteArray, QList<const Chimera *> > _previously_parsed;
@@ -263,16 +265,16 @@ private:
     bool parse_cpp_type(const QByteArray &type);
     bool parse_py_type(PyTypeObject *type_obj);
     sipAssignFunc get_assign_helper() const;
-    void set_flag();
-    bool to_QVariantList(PyObject *py, QVariantList &cpp) const;
-    bool to_QVariantMap(PyObject *py, QVariantMap &cpp) const;
-    bool to_QVariantHash(PyObject *py, QVariantHash &cpp) const;
-    static bool add_variant_to_dict(PyObject *dict, const QString &key_ref,
-            const QVariant &val_ref);
-    static void raiseParseException(const char *type, const char *context);
+    void set_qflags(const sipTypeDef *td);
+    static bool to_char16_t(PyObject *py, char16_t &cpp);
+    static bool to_char32_t(PyObject *py, char32_t &cpp);
+    static void raiseParseCppException(const char *type,
+            const char *context = 0);
     static QVariant keep_as_pyobject(PyObject *py);
     static int extract_raw_type(const QByteArray &type, QByteArray &raw_type);
     static QByteArray resolve_types(const QByteArray &type);
+    int get_enum_value(PyObject *py) const;
+    PyObject *get_enum_key(int cpp) const;
 
     Chimera &operator=(const Chimera &);
 };
