@@ -1,8 +1,8 @@
 // This is the implementation of the Chimera::Signature class.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -24,11 +24,7 @@
 
 
 // Forward declarations.
-#if defined(SIP_USE_PYCAPSULE)
 extern "C" {static void Signature_delete(PyObject *cap);}
-#else
-extern "C" {static void Signature_delete(void *sig);}
-#endif
 
 
 // Destroy the signature and any type objects.
@@ -45,11 +41,7 @@ Chimera::Signature::~Signature()
 // Return the parsed signature wrapped in a Python object.
 PyObject *Chimera::Signature::toPyObject(Chimera::Signature *parsed_signature)
 {
-#if defined(SIP_USE_PYCAPSULE)
     PyObject *py = PyCapsule_New(parsed_signature, NULL, Signature_delete);
-#else
-    PyObject *py = PyCObject_FromVoidPtr(parsed_signature, Signature_delete);
-#endif
 
     if (!py)
         delete parsed_signature;
@@ -61,11 +53,7 @@ PyObject *Chimera::Signature::toPyObject(Chimera::Signature *parsed_signature)
 // Return the parsed signature extracted from a Python object.
 Chimera::Signature *Chimera::Signature::fromPyObject(PyObject *py)
 {
-#if defined(SIP_USE_PYCAPSULE)
     return reinterpret_cast<Chimera::Signature *>(PyCapsule_GetPointer(py, NULL));
-#else
-    return reinterpret_cast<Chimera::Signature *>(PyCObject_AsVoidPtr(py));
-#endif
 }
 
 
@@ -97,17 +85,9 @@ QByteArray Chimera::Signature::arguments(const QByteArray &signature)
 }
 
 
-#if defined(SIP_USE_PYCAPSULE)
 // The PyCapsule destructor for the Chimera::Signature type.
 static void Signature_delete(PyObject *cap)
 {
     delete reinterpret_cast<Chimera::Signature *>(
             PyCapsule_GetPointer(cap, NULL));
 }
-#else
-// The PyCObject destructor for the Chimera::Signature type.
-static void Signature_delete(void *sig)
-{
-    delete reinterpret_cast<Chimera::Signature *>(sig);
-}
-#endif
