@@ -1,8 +1,8 @@
 // This is the implementation of the Chimera::Storage class.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -59,7 +59,10 @@ Chimera::Storage::Storage(const Chimera *ct)
     if (!isPointerType())
     {
         // Create a default fundamental or value type.
-        _value_storage = QVariant(_parsed_type->metatype(), (const void *)0);
+        if (_parsed_type->metatype.id() == QMetaType::Void)
+            _value_storage = QVariant();
+        else
+            _value_storage = QVariant(_parsed_type->metatype);
     }
 }
 
@@ -93,7 +96,10 @@ void *Chimera::Storage::address()
 // Return true if the value is a pointer type.
 bool Chimera::Storage::isPointerType() const
 {
-    return (_parsed_type->metatype() == PyQt_PyObject::metatype &&
+    if (_parsed_type->isEnumOrFlag())
+        return false;
+
+    return (_parsed_type->metatype == QMetaType::fromType<PyQt_PyObject>() &&
             _parsed_type->typeDef());
 }
 
