@@ -1,8 +1,8 @@
 // This contains the implementation of the pyqtMethodProxy type.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -31,6 +31,10 @@
 #include "qpycore_pyqtmethodproxy.h"
 
 
+// The type object.
+PyTypeObject *qpycore_pyqtMethodProxy_TypeObject;
+
+
 // Forward declarations.
 extern "C" {
 static PyObject *pyqtMethodProxy_call(PyObject *self, PyObject *args,
@@ -43,58 +47,22 @@ static void parse_arg(PyObject *args, int arg_nr,
         Chimera::Storage **storage, bool &failed, const char *py_name);
 
 
-// The pyqtMethodProxy type object.
-PyTypeObject qpycore_pyqtMethodProxy_Type = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    SIP_TPNAME_CAST("PyQt5.QtCore.pyqtMethodProxy"),    /* tp_name */
-    sizeof (qpycore_pyqtMethodProxy),   /* tp_basicsize */
-    0,                      /* tp_itemsize */
-    pyqtMethodProxy_dealloc,    /* tp_dealloc */
-    0,                      /* tp_print */
-    0,                      /* tp_getattr */
-    0,                      /* tp_setattr */
-    0,                      /* tp_compare */
-    0,                      /* tp_repr */
-    0,                      /* tp_as_number */
-    0,                      /* tp_as_sequence */
-    0,                      /* tp_as_mapping */
-    0,                      /* tp_hash */
-    pyqtMethodProxy_call,   /* tp_call */
-    0,                      /* tp_str */
-    0,                      /* tp_getattro */
-    0,                      /* tp_setattro */
-    0,                      /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT,     /* tp_flags */
-    0,                      /* tp_doc */
-    0,                      /* tp_traverse */
-    0,                      /* tp_clear */
-    0,                      /* tp_richcompare */
-    0,                      /* tp_weaklistoffset */
-    0,                      /* tp_iter */
-    0,                      /* tp_iternext */
-    0,                      /* tp_methods */
-    0,                      /* tp_members */
-    0,                      /* tp_getset */
-    0,                      /* tp_base */
-    0,                      /* tp_dict */
-    0,                      /* tp_descr_get */
-    0,                      /* tp_descr_set */
-    0,                      /* tp_dictoffset */
-    0,                      /* tp_init */
-    0,                      /* tp_alloc */
-    PyType_GenericNew,      /* tp_new */
-    0,                      /* tp_free */
-    0,                      /* tp_is_gc */
-    0,                      /* tp_bases */
-    0,                      /* tp_mro */
-    0,                      /* tp_cache */
-    0,                      /* tp_subclasses */
-    0,                      /* tp_weaklist */
-    0,                      /* tp_del */
-    0,                      /* tp_version_tag */
-#if PY_VERSION_HEX >= 0x03040000
-    0,                      /* tp_finalize */
-#endif
+// Define the slots.
+static PyType_Slot qpycore_pyqtMethodProxy_Slots[] = {
+    {Py_tp_new,         (void *)PyType_GenericNew},
+    {Py_tp_dealloc,     (void *)pyqtMethodProxy_dealloc},
+    {Py_tp_call,        (void *)pyqtMethodProxy_call},
+    {0,                 0}
+};
+
+
+// Define the type.
+static PyType_Spec qpycore_pyqtMethodProxy_Spec = {
+    "PyQt6.QtCore.pyqtMethodProxy",
+    sizeof (qpycore_pyqtMethodProxy),
+    0,
+    Py_TPFLAGS_DEFAULT,
+    qpycore_pyqtMethodProxy_Slots
 };
 
 
@@ -105,7 +73,7 @@ static void pyqtMethodProxy_dealloc(PyObject *self)
 
     delete mp->py_name;
 
-    Py_TYPE(self)->tp_free(self);
+    PyObject_Del(self);
 }
 
 
@@ -179,8 +147,10 @@ static PyObject *pyqtMethodProxy_call(PyObject *self, PyObject *args,
 
     if (!failed)
     {
-        failed = !method.invoke(mp->qobject, ret, a0, a1, a2, a3, a4, a5, a6,
-                a7, a8, a9);
+        Py_BEGIN_ALLOW_THREADS
+        failed = !method.invoke(mp->qobject, Qt::DirectConnection, ret, a0, a1,
+                a2, a3, a4, a5, a6, a7, a8, a9);
+        Py_END_ALLOW_THREADS
 
         if (failed)
         {
@@ -235,7 +205,7 @@ static void parse_arg(PyObject *args, int arg_nr,
     if (arg_nr >= types.size())
         return;
 
-    PyObject *py_arg = PyTuple_GET_ITEM(args, arg_nr);
+    PyObject *py_arg = PyTuple_GetItem(args, arg_nr);
     const QByteArray &cpp_type = types.at(arg_nr);
 
     const Chimera *ct = Chimera::parse(cpp_type);
@@ -253,7 +223,7 @@ static void parse_arg(PyObject *args, int arg_nr,
 
         PyErr_Format(PyExc_TypeError,
                 "unable to convert argument %d of %s from '%s' to '%s'",
-                arg_nr, py_name, Py_TYPE(py_arg)->tp_name,
+                arg_nr, py_name, sipPyTypeName(Py_TYPE(py_arg)),
                 cpp_type.constData());
 
         failed = true;
@@ -266,6 +236,16 @@ static void parse_arg(PyObject *args, int arg_nr,
 }
 
 
+// Initialise the type and return true if there was no error.
+bool qpycore_pyqtMethodProxy_init_type()
+{
+    qpycore_pyqtMethodProxy_TypeObject = (PyTypeObject *)PyType_FromSpec(
+            &qpycore_pyqtMethodProxy_Spec);
+
+    return qpycore_pyqtMethodProxy_TypeObject;
+}
+
+
 // Create a proxy for a bound introspected method.
 PyObject *qpycore_pyqtMethodProxy_New(QObject *qobject, int method_index,
         const QByteArray &py_name)
@@ -273,7 +253,7 @@ PyObject *qpycore_pyqtMethodProxy_New(QObject *qobject, int method_index,
     qpycore_pyqtMethodProxy *mp;
 
     mp = (qpycore_pyqtMethodProxy *)PyType_GenericAlloc(
-            &qpycore_pyqtMethodProxy_Type, 0);
+            qpycore_pyqtMethodProxy_TypeObject, 0);
 
     if (!mp)
         return 0;
