@@ -1,8 +1,8 @@
 // This contains the implementation of the pyqtSignal type.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -30,6 +30,10 @@
 #include "qpycore_pyqtsignal.h"
 
 
+// The type object.
+PyTypeObject *qpycore_pyqtSignal_TypeObject;
+
+
 // Forward declarations.
 extern "C" {
 static PyObject *pyqtSignal_call(PyObject *self, PyObject *args, PyObject *kw);
@@ -40,6 +44,7 @@ static int pyqtSignal_init(PyObject *self, PyObject *args, PyObject *kwd_args);
 static PyObject *pyqtSignal_repr(PyObject *self);
 static PyObject *pyqtSignal_mp_subscript(PyObject *self, PyObject *subscript);
 static PyObject *pyqtSignal_get_doc(PyObject *self, void *);
+static PyObject *pyqtSignal_get_signatures(PyObject *self, void *);
 }
 
 static int init_signal_from_types(qpycore_pyqtSignal *ps, const char *name,
@@ -49,23 +54,9 @@ static void append_overload(qpycore_pyqtSignal *ps);
 static bool is_signal_name(const char *sig, const QByteArray &name);
 
 
-// Define the mapping methods.
-static PyMappingMethods pyqtSignal_as_mapping = {
-    0,                      /* mp_length */
-    pyqtSignal_mp_subscript,    /* mp_subscript */
-    0,                      /* mp_ass_subscript */
-};
-
-
-// The getters/setters.
-static PyGetSetDef pyqtSignal_getsets[] = {
-    {(char *)"__doc__", pyqtSignal_get_doc, NULL, NULL, NULL},
-    {NULL, NULL, NULL, NULL, NULL}
-};
-
-
+// Doc-strings.
 PyDoc_STRVAR(pyqtSignal_doc,
-"pyqtSignal(*types, name=str, revision=0, arguments=[]) -> signal\n"
+"pyqtSignal(*types, name: str = ..., revision: int = ..., arguments: Sequence = ...) -> PYQT_SIGNAL\n"
 "\n"
 "types is normally a sequence of individual types.  Each type is either a\n"
 "type object or a string that is the name of a C++ type.  Alternatively\n"
@@ -74,61 +65,44 @@ PyDoc_STRVAR(pyqtSignal_doc,
 "name is the optional C++ name of the signal.  If it is not specified then\n"
 "the name of the class attribute that is bound to the signal is used.\n"
 "revision is the optional revision of the signal that is exported to QML.\n"
-"arguments is the option sequence of the names of the signal's arguments.\n");
+"If it is not specified then 0 is used.\n"
+"arguments is the optional sequence of the names of the signal's arguments.\n");
+
+PyDoc_STRVAR(pyqtSignal_signatures_doc,
+"The signatures of each of the overloaded signals");
 
 
-// The pyqtSignal type object.
-PyTypeObject qpycore_pyqtSignal_Type = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    SIP_TPNAME_CAST("PyQt5.QtCore.pyqtSignal"), /* tp_name */
-    sizeof (qpycore_pyqtSignal),    /* tp_basicsize */
-    0,                      /* tp_itemsize */
-    pyqtSignal_dealloc,     /* tp_dealloc */
-    0,                      /* tp_print */
-    0,                      /* tp_getattr */
-    0,                      /* tp_setattr */
-    0,                      /* tp_compare */
-    pyqtSignal_repr,        /* tp_repr */
-    0,                      /* tp_as_number */
-    0,                      /* tp_as_sequence */
-    &pyqtSignal_as_mapping, /* tp_as_mapping */
-    0,                      /* tp_hash */
-    pyqtSignal_call,        /* tp_call */
-    0,                      /* tp_str */
-    0,                      /* tp_getattro */
-    0,                      /* tp_setattro */
-    0,                      /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT,     /* tp_flags */
-    pyqtSignal_doc,         /* tp_doc */
-    0,                      /* tp_traverse */
-    0,                      /* tp_clear */
-    0,                      /* tp_richcompare */
-    0,                      /* tp_weaklistoffset */
-    0,                      /* tp_iter */
-    0,                      /* tp_iternext */
-    0,                      /* tp_methods */
-    0,                      /* tp_members */
-    pyqtSignal_getsets,     /* tp_getset */
-    0,                      /* tp_base */
-    0,                      /* tp_dict */
-    pyqtSignal_descr_get,   /* tp_descr_get */
-    0,                      /* tp_descr_set */
-    0,                      /* tp_dictoffset */
-    pyqtSignal_init,        /* tp_init */
-    0,                      /* tp_alloc */
-    PyType_GenericNew,      /* tp_new */
-    0,                      /* tp_free */
-    0,                      /* tp_is_gc */
-    0,                      /* tp_bases */
-    0,                      /* tp_mro */
-    0,                      /* tp_cache */
-    0,                      /* tp_subclasses */
-    0,                      /* tp_weaklist */
-    0,                      /* tp_del */
-    0,                      /* tp_version_tag */
-#if PY_VERSION_HEX >= 0x03040000
-    0,                      /* tp_finalize */
-#endif
+// The getters/setters.
+static PyGetSetDef pyqtSignal_getset[] = {
+    {const_cast<char *>("__doc__"), pyqtSignal_get_doc, NULL, NULL, NULL},
+    {const_cast<char *>("signatures"), pyqtSignal_get_signatures, NULL,
+            const_cast<char *>(pyqtSignal_signatures_doc), NULL},
+    {NULL, NULL, NULL, NULL, NULL}
+};
+
+
+// Define the slots.
+static PyType_Slot qpycore_pyqtSignal_Slots[] = {
+    {Py_tp_new,         (void *)PyType_GenericNew},
+    {Py_tp_init,        (void *)pyqtSignal_init},
+    {Py_tp_dealloc,     (void *)pyqtSignal_dealloc},
+    {Py_tp_doc,         (void *)pyqtSignal_doc},
+    {Py_tp_repr,        (void *)pyqtSignal_repr},
+    {Py_tp_call,        (void *)pyqtSignal_call},
+    {Py_tp_descr_get,   (void *)pyqtSignal_descr_get},
+    {Py_mp_subscript,   (void *)pyqtSignal_mp_subscript},
+    {Py_tp_getset,      pyqtSignal_getset},
+    {0,                 0}
+};
+
+
+// Define the type.
+static PyType_Spec qpycore_pyqtSignal_Spec = {
+    "PyQt6.QtCore.pyqtSignal",
+    sizeof (qpycore_pyqtSignal),
+    0,
+    Py_TPFLAGS_DEFAULT,
+    qpycore_pyqtSignal_Slots
 };
 
 
@@ -174,13 +148,39 @@ static PyObject *pyqtSignal_get_doc(PyObject *self, void *)
         return Py_None;
     }
 
-    return
-#if PY_MAJOR_VERSION >= 3
-        PyUnicode_FromString
-#else
-        PyString_FromString
-#endif
-            (doc.constData() + 1);
+    return PyUnicode_FromString(doc.constData() + 1);
+}
+
+
+// The 'signatures' getter.
+static PyObject *pyqtSignal_get_signatures(PyObject *self, void *)
+{
+    qpycore_pyqtSignal *head = ((qpycore_pyqtSignal *)self)->default_signal;
+    qpycore_pyqtSignal *ps;
+    PyObject *sigs;
+    Py_ssize_t sig_nr;
+
+    // Count the number of overloads.
+    for (sig_nr = 0, ps = head; ps; ps = ps->next, ++sig_nr)
+        ;
+
+    sigs = PyTuple_New(sig_nr);
+    if (!sigs)
+        return 0;
+
+    for (sig_nr = 0, ps = head; ps; ps = ps->next, ++sig_nr)
+    {
+        PyObject *sig = PyUnicode_FromString(
+                ps->parsed_signature->signature.constData() + 1);
+
+        if (!sig || PyTuple_SetItem(sigs, sig_nr, sig))
+        {
+            Py_DECREF(sigs);
+            return 0;
+        }
+    }
+
+    return sigs;
 }
 
 
@@ -189,13 +189,8 @@ static PyObject *pyqtSignal_repr(PyObject *self)
 {
     qpycore_pyqtSignal *ps = (qpycore_pyqtSignal *)self;
 
-    return
-#if PY_MAJOR_VERSION >= 3
-        PyUnicode_FromFormat
-#else
-        PyString_FromFormat
-#endif
-            ("<unbound signal %s>", ps->parsed_signature->name().constData() + 1);
+    return PyUnicode_FromFormat("<unbound PYQT_SIGNAL %s>",
+            ps->parsed_signature->signature.constData() + 1);
 }
 
 
@@ -231,7 +226,7 @@ static void pyqtSignal_dealloc(PyObject *self)
         }
     }
 
-    Py_TYPE(self)->tp_free(self);
+    PyObject_Del(self);
 }
 
 
@@ -254,12 +249,7 @@ static PyObject *pyqtSignal_descr_get(PyObject *self, PyObject *obj,
             SIP_NO_CONVERTORS, 0, &is_err);
 
     if (is_err)
-    {
-        PyErr_Format(PyExc_TypeError,
-                "pyqtSignal must be bound to a QObject, not '%s'",
-                Py_TYPE(obj)->tp_name);
         return 0;
-    }
 
     // Return the bound signal.
     return qpycore_pyqtBoundSignal_New(ps, obj,
@@ -280,18 +270,12 @@ static int pyqtSignal_init(PyObject *self, PyObject *args, PyObject *kwd_args)
 
     if (kwd_args)
     {
-        SIP_SSIZE_T pos = 0;
+        Py_ssize_t pos = 0;
         PyObject *key, *value;
 
         while (PyDict_Next(kwd_args, &pos, &key, &value))
         {
-#if PY_MAJOR_VERSION >= 3
             if (PyUnicode_CompareWithASCIIString(key, "name") == 0)
-#else
-            Q_ASSERT(PyString_Check(key));
-
-            if (qstrcmp(PyString_AS_STRING(key), "name") == 0)
-#endif
             {
                 name_obj = value;
                 name = sipString_AsASCIIString(&name_obj);
@@ -300,48 +284,39 @@ static int pyqtSignal_init(PyObject *self, PyObject *args, PyObject *kwd_args)
                 {
                     PyErr_Format(PyExc_TypeError,
                             "signal 'name' must be a str, not %s",
-                            Py_TYPE(value)->tp_name);
+                            sipPyTypeName(Py_TYPE(value)));
 
                     return -1;
                 }
             }
-#if PY_MAJOR_VERSION >= 3
             else if (PyUnicode_CompareWithASCIIString(key, "revision") == 0)
-#else
-            else if (qstrcmp(PyString_AS_STRING(key), "revision") == 0)
-#endif
             {
-                PyErr_Clear();
-
-                revision = SIPLong_AsLong(value);
+                revision = sipLong_AsInt(value);
 
                 if (PyErr_Occurred())
                 {
-                    PyErr_Format(PyExc_TypeError,
-                            "signal 'revision' must be an int, not %s",
-                            Py_TYPE(value)->tp_name);
+                    if (PyErr_ExceptionMatches(PyExc_TypeError))
+                        PyErr_Format(PyExc_TypeError,
+                                "signal 'revision' must be an int, not %s",
+                                sipPyTypeName(Py_TYPE(value)));
 
                     Py_XDECREF(name_obj);
                     return -1;
                 }
             }
-#if PY_MAJOR_VERSION >= 3
             else if (PyUnicode_CompareWithASCIIString(key, "arguments") == 0)
-#else
-            else if (qstrcmp(PyString_AS_STRING(key), "arguments") == 0)
-#endif
             {
                 bool ok = true;
 
                 if (PySequence_Check(value))
                 {
-                    SIP_SSIZE_T len = PySequence_Size(value);
+                    Py_ssize_t len = PySequence_Size(value);
 
                     parameter_names = new QList<QByteArray>;
 
-                    for (SIP_SSIZE_T i = 0; i < len; ++i)
+                    for (Py_ssize_t i = 0; i < len; ++i)
                     {
-                        PyObject *py_attr = PySequence_ITEM(value, i);
+                        PyObject *py_attr = PySequence_GetItem(value, i);
 
                         if (!py_attr)
                         {
@@ -349,21 +324,16 @@ static int pyqtSignal_init(PyObject *self, PyObject *args, PyObject *kwd_args)
                             break;
                         }
 
-                        PyObject *py_ascii_attr = py_attr;
-                        const char *attr = sipString_AsASCIIString(
-                                &py_ascii_attr);
-
+                        QByteArray attr = qpycore_convert_ASCII(py_attr);
                         Py_DECREF(py_attr);
 
-                        if (!attr)
+                        if (attr.isNull())
                         {
                             ok = false;
                             break;
                         }
 
-                        parameter_names->append(QByteArray(attr));
-
-                        Py_DECREF(py_ascii_attr);
+                        parameter_names->append(attr);
                     }
                 }
                 else
@@ -375,7 +345,7 @@ static int pyqtSignal_init(PyObject *self, PyObject *args, PyObject *kwd_args)
                 {
                     PyErr_Format(PyExc_TypeError,
                             "signal 'attribute_names' must be a sequence of str, not %s",
-                            Py_TYPE(value)->tp_name);
+                            sipPyTypeName(Py_TYPE(value)));
 
                     if (parameter_names)
                         delete parameter_names;
@@ -386,15 +356,9 @@ static int pyqtSignal_init(PyObject *self, PyObject *args, PyObject *kwd_args)
             }
             else
             {
-#if PY_MAJOR_VERSION >= 3
                 PyErr_Format(PyExc_TypeError,
                         "pyqtSignal() got an unexpected keyword argument '%U'",
                         key);
-#else
-                PyErr_Format(PyExc_TypeError,
-                        "pyqtSignal() got an unexpected keyword argument '%s'",
-                        PyString_AS_STRING(key));
-#endif
 
                 Py_XDECREF(name_obj);
                 return -1;
@@ -405,11 +369,11 @@ static int pyqtSignal_init(PyObject *self, PyObject *args, PyObject *kwd_args)
     // If there is at least one argument and it is a sequence then assume all
     // arguments are sequences.  Unfortunately a string is also a sequence so
     // check for tuples and lists explicitly.
-    if (PyTuple_GET_SIZE(args) > 0 && (PyTuple_Check(PyTuple_GET_ITEM(args, 0)) || PyList_Check(PyTuple_GET_ITEM(args, 0))))
+    if (PyTuple_Size(args) > 0 && (PyTuple_Check(PyTuple_GetItem(args, 0)) || PyList_Check(PyTuple_GetItem(args, 0))))
     {
-        for (SIP_SSIZE_T i = 0; i < PyTuple_GET_SIZE(args); ++i)
+        for (Py_ssize_t i = 0; i < PyTuple_Size(args); ++i)
         {
-            PyObject *types = PySequence_Tuple(PyTuple_GET_ITEM(args, i));
+            PyObject *types = PySequence_Tuple(PyTuple_GetItem(args, i));
 
             if (!types)
             {
@@ -434,7 +398,7 @@ static int pyqtSignal_init(PyObject *self, PyObject *args, PyObject *kwd_args)
             }
             else
             {
-                qpycore_pyqtSignal *overload = (qpycore_pyqtSignal *)PyType_GenericNew(&qpycore_pyqtSignal_Type, 0, 0);
+                qpycore_pyqtSignal *overload = (qpycore_pyqtSignal *)PyType_GenericNew(qpycore_pyqtSignal_TypeObject, 0, 0);
 
                 if (!overload)
                 {
@@ -497,6 +461,16 @@ static PyObject *pyqtSignal_mp_subscript(PyObject *self, PyObject *subscript)
 }
 
 
+// Initialise the type and return true if there was no error.
+bool qpycore_pyqtSignal_init_type()
+{
+    qpycore_pyqtSignal_TypeObject = (PyTypeObject *)PyType_FromSpec(
+            &qpycore_pyqtSignal_Spec);
+
+    return qpycore_pyqtSignal_TypeObject;
+}
+
+
 // Create a new signal instance.
 qpycore_pyqtSignal *qpycore_pyqtSignal_New(const char *signature, bool *fatal)
 {
@@ -504,8 +478,12 @@ qpycore_pyqtSignal *qpycore_pyqtSignal_New(const char *signature, bool *fatal)
     if (fatal)
         *fatal = true;
 
-    Chimera::Signature *parsed_signature = Chimera::parse(signature,
-                "a signal argument");
+    // See if we have two variants of the signature, ie. one with namespaces
+    // stripped that we can use with connect() and another that we can parse.
+    const char *full_sig = strchr(signature, '|');
+
+    Chimera::Signature *parsed_signature = Chimera::parse(
+            (full_sig ? full_sig + 1 : signature), "a signal argument");
 
     // At first glance the parse should never fail because the signature
     // originates from the .sip file.  However it might if it includes a type
@@ -520,11 +498,16 @@ qpycore_pyqtSignal *qpycore_pyqtSignal_New(const char *signature, bool *fatal)
         return 0;
     }
 
+    if (full_sig)
+        // Use the variant with namespaces stripped.
+        parsed_signature->signature = QByteArray(signature,
+                full_sig - signature);
+
     parsed_signature->signature.prepend('2');
 
     // Create and initialise the signal.
     qpycore_pyqtSignal *ps = (qpycore_pyqtSignal *)PyType_GenericNew(
-            &qpycore_pyqtSignal_Type, 0, 0);
+            qpycore_pyqtSignal_TypeObject, 0, 0);
 
     if (!ps)
     {
@@ -563,7 +546,7 @@ qpycore_pyqtSignal *qpycore_find_signal(qpycore_pyqtSignal *ps,
         if (!args)
             return 0;
 
-        PyTuple_SET_ITEM(args, 0, subscript);
+        PyTuple_SetItem(args, 0, subscript);
     }
 
     Py_INCREF(subscript);
@@ -671,10 +654,14 @@ void qpycore_set_signal_name(qpycore_pyqtSignal *ps, const char *type_name,
 // Handle the getting of a lazy attribute, ie. a native Qt signal.
 int qpycore_get_lazy_attr(const sipTypeDef *td, PyObject *dict)
 {
-    pyqt5ClassTypeDef *ctd = (pyqt5ClassTypeDef *)td;
-    const pyqt5QtSignal *sigs = ctd->qt_signals;
+    const pyqt6ClassPluginDef *plugin = reinterpret_cast<const pyqt6ClassPluginDef *>(sipTypePluginData(td));
+
+    // Handle the trvial cases.
+    if (!plugin)
+        return 0;
+
+    const pyqt6QtSignal *sigs = plugin->qt_signals;
 
-    // Handle the trvial case.
     if (!sigs)
         return 0;
 
@@ -758,7 +745,7 @@ PyObject *qpycore_call_signal_overload(qpycore_pyqtSignal *ps, PyObject *bound,
     if (!func)
         return 0;
 
-    PyObject *result = PyCFunction_Call(func, args, kw);
+    PyObject *result = PyObject_Call(func, args, kw);
 
     Py_DECREF(func);
 
