@@ -1,8 +1,8 @@
 // This is the support for QMessageLogger.
 //
-// Copyright (c) 2023 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -76,25 +76,15 @@ int qpycore_current_context(const char **file, const char **function)
         goto py_error;
 
     Py_XDECREF(saved_file);
-#if PY_MAJOR_VERSION >= 3
     saved_file = PyUnicode_AsEncodedString(file_obj, "latin_1", "ignore");
-#else
-    saved_file = file_obj;
-    Py_INCREF(saved_file);
-#endif
-    *file = SIPBytes_AsString(saved_file);
+    *file = PyBytes_AsString(saved_file);
 
     // Ignore any overflow exception.
     linenr = sipLong_AsInt(linenr_obj);
 
     Py_XDECREF(saved_function);
-#if PY_MAJOR_VERSION >= 3
     saved_function = PyUnicode_AsEncodedString(function_obj, "latin_1", "ignore");
-#else
-    saved_function = function_obj;
-    Py_INCREF(saved_function);
-#endif
-    *function = SIPBytes_AsString(saved_function);
+    *function = PyBytes_AsString(saved_function);
 
     Py_DECREF(info);
     Py_DECREF(frame);
@@ -105,7 +95,7 @@ py_error:
     Py_XDECREF(info);
     Py_XDECREF(frame);
 
-    pyqt5_err_print();
+    pyqt6_err_print();
 
     *file = *function = "";
     return 0;
