@@ -1,8 +1,8 @@
 // This is the implementation of the various Chimera helpers.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -143,7 +143,7 @@ static PyObject *from_array_type(const QDBusArgument &arg)
             return 0;
         }
 
-        PyList_SET_ITEM(obj, i, itm);
+        PyList_SetItem(obj, i, itm);
     }
 
     return obj;
@@ -177,7 +177,7 @@ static PyObject *from_structure_type(const QDBusArgument &arg)
             return 0;
         }
 
-        PyTuple_SET_ITEM(obj, i, itm);
+        PyTuple_SetItem(obj, i, itm);
     }
 
     return obj;
