@@ -1,9 +1,9 @@
 // This is the definition and implementation of the
 // QPyDesignerCustomWidgetPlugin class.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -24,7 +24,7 @@
 
 
 #include <QObject>
-#include <QDesignerCustomWidgetInterface>
+#include <QtDesigner>
 
 
 class QPyDesignerCustomWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
