@@ -1,8 +1,8 @@
 // This is the implementation of the ListData class.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -18,6 +18,8 @@
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
 
+#include <sip.h>
+
 #include "qpyqml_listdata.h"
 
 
@@ -42,6 +44,8 @@ ListData::ListData(PyObject *py_type_, PyObject *py_obj_, PyObject *py_list_,
 // Destroy the data.
 ListData::~ListData()
 {
+    SIP_BLOCK_THREADS
+
     Py_XDECREF(py_type);
     Py_XDECREF(py_obj);
     Py_XDECREF(py_list);
@@ -49,4 +53,6 @@ ListData::~ListData()
     Py_XDECREF(py_count);
     Py_XDECREF(py_at);
     Py_XDECREF(py_clear);
+
+    SIP_UNBLOCK_THREADS
 }
