@@ -1,8 +1,8 @@
 // This contains the main implementation of qmlRegisterSingletonType.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -53,9 +53,11 @@ int qpyqml_register_singleton_type(PyTypeObject *py_type, const char *uri,
     if (!rt)
         return -1;
 
+    if (!type_name)
+        type_name = sipPyTypeName(py_type);
+
     rt->uri = uri;
-    rt->versionMajor = major;
-    rt->versionMinor = minor;
+    rt->version = QTypeRevision::fromVersion(major, minor);
     rt->typeName = type_name;
 
     return register_type(rt);
@@ -82,9 +84,8 @@ static int register_type(QQmlPrivate::RegisterSingletonType *rt)
     case n##U: \
         QPyQmlSingletonObject##n::staticMetaObject = *mo; \
         QPyQmlSingletonObject##n::factory = factory; \
-        rt->qobjectApi = QPyQmlSingletonObject##n::objectFactory; \
-        rt->typeId = qRegisterNormalizedMetaType<QPyQmlSingletonObject##n *>(ptr_name.constData()); \
-        rt->instanceMetaObject = mo; \
+        rt->qObjectApi = QPyQmlSingletonObject##n::objectFactory; \
+        rt->typeId = QQmlPrivate::QmlMetaType<QPyQmlSingletonObject##n>::self(); \
         break
 
 
@@ -113,9 +114,9 @@ static QQmlPrivate::RegisterSingletonType *init_type(PyTypeObject *py_type,
 
     QQmlPrivate::RegisterSingletonType *rt = &proxy_types[type_nr];
 
-    const QMetaObject *mo = pyqt5_qtqml_get_qmetaobject(py_type);
+    const QMetaObject *mo = pyqt6_qtqml_get_qmetaobject(py_type);
 
-    QByteArray ptr_name(py_type->tp_name);
+    QByteArray ptr_name(sipPyTypeName(py_type));
     ptr_name.append('*');
 
     // Initialise those members that depend on the C++ type.
@@ -154,13 +155,14 @@ static QQmlPrivate::RegisterSingletonType *init_type(PyTypeObject *py_type,
     }
 
     // Initialise the remaining members.
-    rt->version = 2;
+    rt->instanceMetaObject = mo;
+
+    rt->structVersion = 0;
     rt->uri = 0;
-    rt->versionMajor = 0;
-    rt->versionMinor = 0;
+    rt->version = QTypeRevision::zero();
     rt->typeName = 0;
     rt->scriptApi = 0;
-    rt->revision = 0;
+    rt->revision = QTypeRevision::zero();
 
     Py_INCREF(factory);
 
