@@ -1,8 +1,8 @@
 // This is the implementation of the QPyQuickPaintedItem classes.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2025 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -36,12 +36,16 @@ static QList<PyTypeObject *> pyqt_types;
 // The registration data for the canned types.
 static QQmlPrivate::RegisterType canned_types[NrOfQuickPaintedItemTypes];
 
+// External declarations.
+extern const QMetaObject *qpyquick_pick_metaobject(const QMetaObject *super_mo,
+        const QMetaObject *static_mo);
+
 
 #define QPYQUICKPAINTEDITEM_INIT(n) \
     case n##U: \
         QPyQuickPaintedItem##n::staticMetaObject = *mo; \
-        rt->typeId = qRegisterNormalizedMetaType<QPyQuickPaintedItem##n *>(ptr_name.constData()); \
-        rt->listId = qRegisterNormalizedMetaType<QQmlListProperty<QPyQuickPaintedItem##n> >(list_name.constData()); \
+        rt->typeId = QQmlPrivate::QmlMetaType<QPyQuickPaintedItem##n>::self(); \
+        rt->listId = QQmlPrivate::QmlMetaType<QPyQuickPaintedItem##n>::list(); \
         rt->objectSize = sizeof(QPyQuickPaintedItem##n); \
         rt->create = QQmlPrivate::createInto<QPyQuickPaintedItem##n>; \
         rt->metaObject = mo; \
@@ -129,7 +133,7 @@ void QPyQuickPaintedItem::createPyObject(QQuickItem *parent)
             NULL, &sipPySelf, "D", parent, sipType_QQuickItem, NULL);
 
     if (!obj)
-        PyErr_Print();
+        pyqt6_qtquick_err_print();
 
     SIP_UNBLOCK_THREADS
 }
@@ -141,6 +145,10 @@ QPyQuickPaintedItem##n::QPyQuickPaintedItem##n(QQuickItem *parent) : QPyQuickPai
 { \
     createPyObject(parent); \
 } \
+const QMetaObject *QPyQuickPaintedItem##n::metaObject() const \
+{ \
+    return qpyquick_pick_metaobject(QPyQuickPaintedItem::metaObject(), &staticMetaObject); \
+} \
 QMetaObject QPyQuickPaintedItem##n::staticMetaObject
 
 
